#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <math.h>
#include <stdbool.h>
#include <ncurses.h>
#include <unistd.h>
#include <termios.h>
#include <limits.h>
#include <time.h>
#include <SDL2/SDL.h>
#include <SDL2/SDL_image.h>

#include "consts.h"
#include "base.h"
#include "edit.h"

void link_cells(cell* parent, cell* child) {
    if(parent != NULL) {
        if(parent->next_1 == NULL) {
            parent->next_1 = child;
        } else if(parent->next_2 == NULL) {
            parent->next_2 = child;
        } else if(parent->next_3 == NULL) {
            parent->next_3 = child;
        } else if(parent->next_4 == NULL) {
            parent->next_4 = child;
        } else {
            fprintf(stderr, "Unable to link cell : parent is at full capacity.\n");
        }
    };
    if(child != NULL) {
        if(child->prev_1 == NULL) {
            child->prev_1 = parent;
        } else if(child->prev_2 == NULL) {
            child->prev_2 = parent;
        } else if(child->prev_3 == NULL) {
            child->prev_3 = parent;
        } else if(child->prev_4 == NULL) {
            child->prev_4 = parent;
        } else {
            fprintf(stderr, "Unable to link cell : child is at full capacity.\n");
        }
    }
}

void unlink_cells(cell* parent, cell* child) {
    if(parent != NULL) {
        if(parent->next_1 == child) {
            parent->next_1 = NULL;
        } else if(parent->next_2 == child) {
            parent->next_2 = NULL;
        } else if(parent->next_3 == child) {
            parent->next_3 = NULL;
        } else if(parent->next_4 == child) {
            parent->next_4 = NULL;
        } else {
            fprintf(stderr, "Unable to unlink cell : no link found.\n");
        }
    };
    if(child != NULL) {
        if(child->prev_1 == parent) {
            child->prev_1 = NULL;
        } else if(child->prev_2 == parent) {
            child->prev_2 = NULL;
        } else if(child->prev_3 == parent) {
            child->prev_3 = NULL;
        } else if(child->prev_4 == parent) {
            child->prev_4 = NULL;
        } else {
            fprintf(stderr, "Unable to unlink cell : no link found.\n");
        }
    };
}

void append_cell(int cell_type, int X, int Y, cell* parent) {
    cell* new = malloc(sizeof(cell));
    new->id = parent->id;        // mostly for parkour
    new->coord_x = X;
    new->coord_y = Y;
    new->star_allowed = is_star_allowed(cell_type);
    new->type = (CELLTYPE)cell_type ;
    new->prev_1 = NULL ;
    new->prev_2 = NULL;
    new->prev_3 = NULL;
    new->prev_4 = NULL;
    new->next_1 = NULL;
    new->next_2 = NULL;
    new->next_3 = NULL;
    new->next_4 = NULL;
    char* meta = "none";

    link_cells(parent, new);
    cell_count[cell_type] += 1;
}

void destroy_cell(cell* cl) {
    unlink_cells(cl, cl->next_1);
    unlink_cells(cl, cl->next_2);
    unlink_cells(cl, cl->next_3);
    unlink_cells(cl, cl->next_4);

    unlink_cells(cl->prev_1, cl);
    unlink_cells(cl->prev_2, cl);
    unlink_cells(cl->prev_3, cl);
    unlink_cells(cl->prev_4, cl);

    cell_count[(int)cl->type] -= 1;

    free(cl);
}